/* This is "bisect1.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program uses the bisection method to solve Equation 9.7 from
   the text.  It prompts the user for the positive and negative
   guesses needed to start off the bisection method. */


#include <stdio.h>
#include <math.h>



/* Old MacDonald's cow function from Chapter 9.  Its roots are the
   same as the roots of Equation 9.7. */

double cow (double theta)
{
  return(sin(theta) - theta*cos(theta) - 3.141592654/2.0);
}



/* Uses the bisection method to find an approximate root for "f",
   where f(pos) > 0 and f(neg) < 0.  Note that "f" is a function that
   takes a double as a parameter and returns a double as its
   result. */

double bisection (double f (double), double pos, double neg)
{

  double avg;   /* Keeps track of the average of "pos" and "neg". */

  /* We initialize "avg" to the value of "pos" rather than to the
     average of "pos" and "neg" so that the while loop that follows
     will not terminate immediately.  (We could have equally well
     initialized "avg" to the value of "neg", or for that matter to
     any value other than the average of "pos" and "neg".) */

  avg = pos;
  

  /* Each time through the loop we set "avg" to the average of "pos"
     and "neg" and then update either "pos" or "neg" according to the
     value of f(avg).  The loop terminates when "avg" remains
     unchanged from one iteration to the next, at which point we
     return "avg" as an approximation to the root of "f". */

  while (avg != (pos+neg)/2.0) {
    avg = (pos + neg) / 2.0;
    if (f(avg) >= 0) {
      pos = avg;
    }
    else {
      neg = avg;
    }
  }
  
  return(avg);
}



/* Prompts for the positive and negative approximations to the root of
   the function cow defined above, then computes and displays an
   approximation to cow's root. */

void main ()
{

  double positive,    /* "Positive" guess, meaning cow(positive) > 0 */
         negative,    /* "Negative" guess, meaning cow(negative) < 0 */
         root;        /* Approximate root of "cow". */

  printf("Enter positive guess: ");
  scanf("%lf", &positive);
  printf("Enter negative guess: ");
  scanf("%lf", &negative);

  root = bisection(cow, positive, negative);

  printf("The root is %.10g\n", root);

}
